################################################################################
## 
## Main script for reproducing
## 
## World Bank Working Paper Series 10822
##
## Predicting Species Extinction Threats using Occurrence Data from the 
## Global Biodiversity Information Facility
## 
## June 2024, revised September 2024
##
## Authors: Susmita Dasgupta, Brian Blankespoor, David Wheeler
##
################################################################################

get_current_file_path <- function(){
     this_file <- grep("^--file=", commandArgs(), value = TRUE)
     this_file <- gsub("^--file=", "", this_file)
     if (length(this_file) == 0) this_file <- rstudioapi::getSourceEditorContext()$path
        return(dirname(this_file))
      }

# set working directory
wkdir <- get_current_file_path()
print(wkdir)
setwd(wkdir)

print(Sys.time())

source(file.path(wkdir,'biod_threats_wp_global_libraries.R'))

## Turn off sf's spherical ("S2") backend
sf_use_s2(FALSE) 
today <- Sys.Date()

#ddh_vers <- '2024-02-19'
ddh_vers <- '2024-08-12'
iucn_vers <- '2022-2'
run_vers <- '2024-09-17' #Sys.Date()

# renv
# renv::init()
tbldir <- file.path(wkdir,'tables_rp',sprintf("vers%s",ddh_vers)) # reproduced tables
figdir <- file.path(wkdir,'figures_rp',sprintf("vers%s",ddh_vers)) # reproduced figures
ddhdir <- file.path(wkdir,'local_data',sprintf("vers%s",ddh_vers)) # ddh version directory
xfun::dir_create(tbldir)
xfun::dir_create(figdir)
xfun::dir_create(ddhdir)

# load gridded data publicly from DDH to ddhdir
ncores = 12
source(file.path(wkdir,'biod_threats_wp_load_gridded_data_ddh.R'))
print("loading gridded species data")

# load boundaries for maps
source(file.path(wkdir,'biod_threats_wp_load_boundary_data.R'))
print("loading boundary data")

# download IUCN red list data publicly from GBIF
source(file.path(wkdir,'biod_threats_wp_load_iucn_species_level_data.R'))
print("loading species red list data : species_tab")

# download data publicly from DDH to ddhdir, species_threats_fn
source(file.path(wkdir,'biod_threats_wp_load_species_level_data.R'))
print("loading species data : species_tab")

##
## tables
##

# table 1 Correlations between predictive model variables
# table 2 Ordered logit results for IUCN Red List categories (former P9 script)
# construct data
source(file.path(wkdir,'biod_threats_wp_tbl01_corr_vars_and_tbl02_ordered_logit_results.R'))

# table 3 : Model-predicted risk assignments vs. IUCN assignments
source(file.path(wkdir,'biod_threats_wp_tbl03_model_predicted_risk_assignments_vs_IUCN_assignments.R'))

# table 4 : Species Occurrence Region sizes of top-ranking species in the model-based composite indicator
source(file.path(wkdir,'biod_threats_wp_tbl04_range_sizes_of_top-ranking_species_tprob.R'))


##
## figures
##

master_threats_csv_fn <- species_tab_fn
# Figure 1 - Terrestrial species with maximum occurrence reports by population group
source(file.path(wkdir,'biod_threats_wp_fig01_terrestrial_species_with_maxcut_occurrence_reports_by_pop_groups.R'))

# Figure 2 : Plants and vertebrates: global cell counts for species in IUCN categories NT, VU, EN, CR 
source(file.path(wkdir,'biod_threats_wp_fig02_plants_and_vertebrates_iucn_match_count.R'))

# Figure 3: Plants and vertebrates: global cell totals for model-predicted threat probabilities, IUCN species match
source(file.path(wkdir,'biod_threats_wp_fig03_plants_and_vertebrates_iucn_match_tprob.R'))

# Figure 4 : Plants and vertebrates: global cell totals for model-predicted threat probabilities, all GBIF Species
source(file.path(wkdir,'biod_threats_wp_fig04_plants_and_vertebrates_gbif_all_tprob.R'))

# Figure 5: Plants and vertebrates: difference in model-predicted threat probabilities, 
# IUCN match vs all GBIF species
# requires output from Figure 3 and Figure 4
source(file.path(wkdir,'biod_threats_wp_fig05_Normalized_GBIF_Tprob_Differences.R'))

# Figure 6: LC assignment vs species occurrence region scale
source(file.path(wkdir,'biod_threats_wp_fig06_LC_assignment_vs_range_scale.R'))

# Figure 7: Plants and vertebrates: Global cell totals for model predictions by threat group
source(file.path(wkdir,'biod_threats_wp_fig07_plants_and_vertebrates_all_species_tprob_by_threat_group.R'))

# Figure 8: Plants and vertebrates: Global cell totals for model-predictions after
# priority reassignment for small species occurrence region species
source(file.path(wkdir,'biod_threats_wp_fig08_plants_and_vertebrates_all_species_tprob_stacked_by_threat_group.R'))

# Figure 9: Plants and vertebrates: Western South America: Effect of priority reassignment 
# for small species occurrence region species
source(file.path(wkdir,'biod_threats_wp_fig09_plants_and_vertebrates_all_species_western_south_america_by_threat_group.R'))

# Figure 10: Plants and vertebrates: differences in global cell totals for model-predictions a
# after priority reassignment for small species occurrence region species
source(file.path(wkdir,'biod_threats_wp_fig10_GBIF_Plants_Vertebrates_Dif_Tprob_vs_stackedAB.R'))

# Figure 11: Plants and vertebrates: five threat indicators
source(file.path(wkdir,'biod_threats_wp_fig11_GBIF_Plants_Vertebrates_five_threat_indicators.R'))


print(Sys.time())