###################################################################
##
## Figure 11: Plants and vertebrates: five threat indicators
## 
###################################################################

##
## Input
##

##
## use boundary data
## source(file.path(wkdir,'biod_threats_wp_load_boundary_data.R'))
##

##
## use gridded species data
## source(file.path(wkdir,'biod_threats_wp_load_gridded_data_ddh.R'))
##

# PROJ.4 string for Eckert IV projection
eckertIV <- "+proj=eck4 +lon_0=0 +x_0=0 +y_0=0 +ellps=WGS84 +datum=WGS84 +units=m +no_defs" 

n<-90
s<--90
w<--180
e<-180
globe0<-pgirmess::bbox2sf(n,s,e,w)

##
## Create data for Figure 11
## extract Plants and Vertebrates Only
##

# load species level data
species_tab_fn <- file.path(wkdir,'local_data',sprintf("vers%s",ddh_vers), "GBIF_Merged_IUCN_Risks_Tab.csv")
species_tab <- data.table::fread(species_tab_fn) %>%
  dplyr::select(species,order,grp,irisk,rcat) %>% 
  rename(group = grp) %>%
  distinct() 
dim(species_tab)

# load species level factors
#xmname=file.path(wkdir,'local_data',sprintf("vers%s",ddh_vers),paste0("GBIF_Species_Threats_Factors",run_vers,".csv"))
xmname=file.path(wkdir,'local_data',sprintf("vers%s",ddh_vers),paste0("GBIF_Species_Threats_Factors",Sys.Date(),".csv"))
xmcmb <- data.table::fread(xmname) %>% distinct()
dim(xmcmb)

species_master <- merge(species_tab,xmcmb,by=c('species'),all.x=TRUE) %>% distinct()

print(sprintf("Species without IUCN rating: %s",sum(is.na(species_master$irisk))))
print(sprintf("Species with IUCN rating: %s",sum(!is.na(species_master$irisk))))

# fixed no Animals_Other in species group : Plants and Vertebrates
spp_groups <- c('Plants_Asparagales','Plants_Asterales','Plants_Caryophyllales','Plants_Ericales',
                'Plants_Fabales','Plants_Gentianales','Plants_Lamiales','Plants_Malpighiales',
                'Plants_Myrtales','Plants_Other','Plants_Poales','Amphibians','Birds',
                'Fish','Mammals','Reptiles')

# summarize across factors at species level
species_tprobs <- species_master %>%
  # extract Plants and Vertebrates Only
  dplyr::filter(group %in% spp_groups) %>%
  # exclude data where all factors are NA
  dplyr::filter(!if_all(c(nfactor1,nfactor2,nfactor3,nfactor4), is.na)) %>%
  dplyr::group_by(species) %>%
  rowwise() %>%
  mutate(factor_max = max(nfactor1,nfactor2,nfactor3,nfactor4,na.rm=TRUE),
         factor_median = median(c(nfactor1,nfactor2,nfactor3,nfactor4),na.rm=TRUE),
         factor_mean = mean(c(nfactor1,nfactor2,nfactor3,nfactor4),na.rm=TRUE),
         range_scale = ifelse(!is.na(nfactor1),nfactor1,NA),
         tprob = ifelse(!is.na(tprob),tprob * 100,NA)) %>%
  dplyr::select(species,factor_max,factor_median,factor_mean,range_scale,tprob) %>%
  # filter out missing across panel
  dplyr::filter(if_all(c(tprob,range_scale,factor_max,factor_median,factor_mean), complete.cases))

# subset for plants and vertebrates, need to join group
# factors have 100
species_plt_vert <- species_tprobs

spp_risks_full_tprob <- species_plt_vert %>% 
  dplyr::filter(!is.na(tprob)) %>%
  as.data.table()
dim(spp_risks_full_tprob)

# Convert to data.table, efficient
dt <- data.table(totresult)

totresult_sub <- dt[species %in% spp_risks_full_tprob$species]
spp_obs_full_tprob <- totresult_sub[spp_risks_full_tprob,on="species",allow.cartesian=TRUE]

# Panel A
# tprob adjusted 0-100
spp_irisk_full_plants_vertebrates_tprob <- spp_obs_full_tprob %>%
  dplyr::filter(!is.na(value)) %>%
  dplyr::select(value,tprob) %>%
  dplyr::arrange(value) %>%
  dplyr::group_by(value) %>%
  dplyr::summarize(species_sum_gbif = sum(tprob,na.rm=TRUE)) %>%
  ungroup() %>%
  dplyr::mutate(max_species_gbif_sum = max(species_sum_gbif,na.rm=T)) %>%
  dplyr::group_by(value) %>%
  dplyr::mutate(species_tprob_index = (species_sum_gbif / max_species_gbif_sum) * 100) %>%
  dplyr::select(value,species_tprob_index) %>%
  as.data.frame()

r11a <- terra::rast(resolution = 0.25)
r11a[spp_irisk_full_plants_vertebrates_tprob[,1]] <- spp_irisk_full_plants_vertebrates_tprob[,2]
r11a_fn <- file.path(wkdir,'local_data',sprintf("vers%s",ddh_vers),
                     paste0('fig_11a_Normalized_Plants_Vertebrates_Species_Tprob',
                            '_ddh',ddh_vers,
                            'run',run_vers,'.tif'))
names(r11a)<-'spp_plt_vert_tprob'
names(r11a) <- 'biod_tprob'
terra::writeRaster(r11a,r11a_fn,overwrite=TRUE)

# clean-up
rm(spp_irisk_full_plants_vertebrates_tprob)

# Panel B
# risk factors are 0-100
spp_irisk_full_plants_vertebrates_tprob_drange <- spp_obs_full_tprob %>%
  dplyr::filter(!is.na(value)) %>%
  dplyr::select(value,range_scale) %>%
  dplyr::arrange(value) %>%
  dplyr::group_by(value) %>%
  dplyr::summarize(species_sum_range_scale = sum(range_scale,na.rm=TRUE)) %>%
  ungroup() %>%
  dplyr::mutate(max_species_sum_range_scale = max(species_sum_range_scale,na.rm=T)) %>%
  dplyr::group_by(value) %>%
  dplyr::mutate(species_tprob_drange = (species_sum_range_scale / max_species_sum_range_scale) * 100) %>%
  dplyr::select(value,species_tprob_drange)  %>%
  as.data.frame()

r11b <- terra::rast(resolution = 0.25)
r11b[spp_irisk_full_plants_vertebrates_tprob_drange[,1]] <- spp_irisk_full_plants_vertebrates_tprob_drange[,2]
r11b_fn <- file.path(wkdir,'local_data',sprintf("vers%s",ddh_vers),
                     paste0('fig_11b_Normalized_Plants_Vertebrates_Species_DifRange',
                            '_ddh',ddh_vers,
                            'run',run_vers,'.tif'))
names(r11b)<-'spp_plt_vert_tprob_drange'
names(r11b) <- 'biod_tprob_drange'
terra::writeRaster(r11b,r11b_fn,overwrite=TRUE)

# clean-up
rm(spp_irisk_full_plants_vertebrates_tprob_drange)

# Panel C
spp_irisk_full_plants_vertebrates_tprob_max <- spp_obs_full_tprob %>%
  dplyr::filter(!is.na(value)) %>%
  dplyr::select(value,factor_max) %>%
  dplyr::arrange(value) %>%
  dplyr::group_by(value) %>%
  dplyr::summarize(species_sum_factor_max = sum(factor_max,na.rm=TRUE)) %>%
  ungroup() %>%
  dplyr::mutate(max_species_sum_factor_max = max(species_sum_factor_max,na.rm=T)) %>%
  dplyr::group_by(value) %>%
  dplyr::mutate(species_factor_max = (species_sum_factor_max / max_species_sum_factor_max) * 100) %>%
  dplyr::select(value,species_factor_max) %>%
  as.data.frame()

dim(spp_irisk_full_plants_vertebrates_tprob_max)

r11c <- terra::rast(resolution = 0.25)
r11c[spp_irisk_full_plants_vertebrates_tprob_max[,1]] <- spp_irisk_full_plants_vertebrates_tprob_max[,2]
r11c_fn <- file.path(wkdir,'local_data',sprintf("vers%s",ddh_vers),
                     paste0('fig_11c_Normalized_Plants_Vertebrates_Species_Max',
                            '_ddh',ddh_vers,
                            'run',run_vers,'.tif'))
names(r11c)<-'spp_plt_vert_tprob_max'
names(r11c) <- 'biod_tprob_max'
terra::writeRaster(r11c,r11c_fn,overwrite=TRUE)

# clean-up
rm(spp_irisk_full_plants_vertebrates_tprob_max)


# Panel D
spp_irisk_full_plants_vertebrates_tprob_median <- spp_obs_full_tprob %>%
  dplyr::filter(!is.na(value)) %>%
  dplyr::select(value,factor_median) %>%
  dplyr::arrange(value) %>%
  dplyr::group_by(value) %>%
  dplyr::summarize(species_sum_factor_median = sum(factor_median,na.rm=TRUE)) %>%
  ungroup() %>%
  dplyr::mutate(max_species_sum_factor_median = max(species_sum_factor_median,na.rm=T)) %>%
  dplyr::group_by(value) %>%
  dplyr::mutate(species_factor_median = (species_sum_factor_median / max_species_sum_factor_median) * 100) %>%
  dplyr::select(value,species_factor_median) %>%
  as.data.frame()

r11d <- terra::rast(resolution = 0.25)
r11d[spp_irisk_full_plants_vertebrates_tprob_median[,1]] <- spp_irisk_full_plants_vertebrates_tprob_median[,2]
r11d_fn <- file.path(wkdir,'local_data',sprintf("vers%s",ddh_vers),
                     paste0('fig_11d_Normalized_Plants_Vertebrates_Species_Median',
                            '_ddh',ddh_vers,
                            'run',run_vers,'.tif'))
names(r11d)<-'spp_plt_vert_tprob_median'
names(r11d) <- 'biod_tprob_median'
terra::writeRaster(r11d,r11d_fn,overwrite=TRUE)

rm(spp_irisk_full_plants_vertebrates_tprob_median)

# Panel E
spp_irisk_full_plants_vertebrates_tprob_mean <- spp_obs_full_tprob %>%
  dplyr::filter(!is.na(value)) %>%
  dplyr::select(value,factor_mean) %>%
  dplyr::arrange(value) %>%
  dplyr::group_by(value) %>%
  dplyr::summarize(species_sum_factor_mean = sum(factor_mean,na.rm=TRUE)) %>%
  ungroup() %>%
  dplyr::mutate(max_species_sum_factor_mean = max(species_sum_factor_mean,na.rm=T)) %>%
  dplyr::group_by(value) %>%
  dplyr::mutate(species_factor_mean = (species_sum_factor_mean / max_species_sum_factor_mean) * 100) %>%
  dplyr::select(value,species_factor_mean) %>%
  as.data.frame()

r11e <- terra::rast(resolution = 0.25)
r11e[spp_irisk_full_plants_vertebrates_tprob_mean[,1]] <- spp_irisk_full_plants_vertebrates_tprob_mean[,2]
r11e_fn <- file.path(wkdir,'local_data',sprintf("vers%s",ddh_vers),
                     paste0('fig_11e_Normalized_Plants_Vertebrates_Species_Mean',
                            '_ddh',ddh_vers,
                            'run',run_vers,'.tif'))
names(r11e)<-'spp_plt_vert_tprob_mean'
names(r11e) <- 'biod_tprob_mean'
terra::writeRaster(r11e,r11e_fn,overwrite=TRUE)

# clean-up
rm(spp_irisk_full_plants_vertebrates_tprob_mean)


# stack all five
stk <- c(r11a,r11b,r11c,r11d,r11e)
names(stk)

##
## Stack as attributes
##

# Projection stack
require(stars)

# split bands to attributes
biod_alloc_stars <- st_as_stars(stk,as_attributes = TRUE)
attributes(biod_alloc_stars)

# reproject
biod_alloc_stars_prj <- st_transform(biod_alloc_stars,crs=eckertIV)
st_bbox(biod_alloc_stars_prj)

# warp so it's no curvilinear
#require(FNN)
biod_alloc_stars_prj = st_warp(biod_alloc_stars_prj, crs = eckertIV,
                               cellsize = 20000,threshold=25000,
                               method = "near")

##
## breaks 
##
temp <- c(biod_alloc_stars_prj$biod_tprob,biod_alloc_stars_prj$biod_tprob_drange,
          biod_alloc_stars_prj$biod_tprob_max,biod_alloc_stars_prj$biod_tprob_median,
          biod_alloc_stars_prj$biod_tprob_mean) %>% 
  as.numeric()

temp <- temp[!is.na(temp) & temp>0]
temp_decile = cut(temp, breaks=c(quantile(temp, probs = seq(0, 1, by = 0.10))))
table(temp_decile)
temp_decile_breaks <- sapply(strsplit(levels(temp_decile), "\\(|,|]"), function(x) as.numeric(x[-1]))
vBreaks <- c(temp_decile_breaks[1,1],temp_decile_breaks[2,])

##
## numeric to factor for maps
##
biod_labels = c("1st","2nd","3rd","4th","5th","6th","7th","8th","9th","10th")

# 1.
biod_alloc_stars_prj$fbiod_tprob <- cut(biod_alloc_stars_prj$biod_tprob,
                                                      breaks=vBreaks,
                                                      labels = biod_labels,
                                                      include.lowest=TRUE, right=TRUE)
biod_alloc_stars_prj$fbiod_tprob <- factor(biod_alloc_stars_prj$fbiod_tprob,
                                                         levels = biod_labels,
                                                         labels = biod_labels)
table(biod_alloc_stars_prj$fbiod_tprob)


# 2. 
biod_alloc_stars_prj$fbiod_tprob_drange <- cut(biod_alloc_stars_prj$biod_tprob_drange, 
                                                      vBreaks,
                                                      labels = biod_labels,
                                                      include.lowest=TRUE, right=TRUE)
biod_alloc_stars_prj$fbiod_tprob_drange <- factor(biod_alloc_stars_prj$fbiod_tprob_drange,
                                                         levels = biod_labels,
                                                         labels = biod_labels)
table(biod_alloc_stars_prj$fbiod_tprob_drange)

# 3. 
biod_alloc_stars_prj$fbiod_tprob_max <- cut(biod_alloc_stars_prj$biod_tprob_max,
                                                      breaks=vBreaks,
                                                      labels = biod_labels,
                                                      include.lowest=TRUE, right=TRUE)
biod_alloc_stars_prj$fbiod_tprob_max <- factor(biod_alloc_stars_prj$fbiod_tprob_max,
                                                         levels = biod_labels,
                                                         labels = biod_labels)
table(biod_alloc_stars_prj$fbiod_tprob_max)

# 4. 
biod_alloc_stars_prj$fbiod_tprob_median <- cut(biod_alloc_stars_prj$biod_tprob_median,
                                            breaks=vBreaks,
                                            labels = biod_labels,
                                            include.lowest=TRUE, right=TRUE)
biod_alloc_stars_prj$fbiod_tprob_median <- factor(biod_alloc_stars_prj$fbiod_tprob_median,
                                               levels = biod_labels,
                                               labels = biod_labels)
table(biod_alloc_stars_prj$fbiod_tprob_median)

# 5. 
biod_alloc_stars_prj$fbiod_tprob_mean <- cut(biod_alloc_stars_prj$biod_tprob_mean,
                                               breaks=vBreaks,
                                               labels = biod_labels,
                                               include.lowest=TRUE, right=TRUE)
biod_alloc_stars_prj$fbiod_tprob_mean <- factor(biod_alloc_stars_prj$fbiod_tprob_mean,
                                                  levels = biod_labels,
                                                  labels = biod_labels)
table(biod_alloc_stars_prj$fbiod_tprob_mean)

biod_hex <- c("#EBEBEB",RColorBrewer::brewer.pal(9,"YlOrRd"))

windowsFonts("Arial" = windowsFont("Arial"))
## theme from 2022-05-11, modified for R&R
## https://www.r-craft.org/r-news/how-to-create-a-beautiful-map-with-r-and-geom_sf-part-2/
mytheme <- theme(text = element_text(family = 'Arial')
                 ,panel.background = element_rect(fill = 'white')
                 ,plot.title = element_text(size = 32)
                 ,plot.subtitle = element_text(size = 14)
                 ,axis.title = element_blank()
                 ,axis.text = element_blank()
)

# subset factor attributes
biod_alloc_stars_prj_sub <- biod_alloc_stars_prj[c("fbiod_tprob","fbiod_tprob_drange",
                                                   "fbiod_tprob_max","fbiod_tprob_median",
                                                   "fbiod_tprob_mean")]

# figure ready names
names(biod_alloc_stars_prj_sub) <- 
  c('Panel a: Model-Predicted',
    'Panel b: Range scale',
    'Panel c: Max Index',
    'Panel d: Median Index',
    'Panel e: Mean Index')

# From attributes to dimension
biod_alloc_stars_prj_sub_re = st_redimension(biod_alloc_stars_prj_sub)
names(biod_alloc_stars_prj_sub_re) = "value"
table(biod_alloc_stars_prj_sub_re$value)
biod_alloc_stars_prj_sub_re
biod_alloc_stars_prj_sub_re$group <- names(biod_alloc_stars_prj_sub)


lineT<- c("countries"="solid", "disputed" = "dotdash","ocean" = "solid")
table(bnd0$bndtype)
table(bnd0$colortype)

# Define custom labeller function
custom_labeller <- function(variable, value) {
  value
}

# plot... models with same legend
p11panel <- ggplot() +
  # alpha transparency parameter for smoothing curvilinear polygons
  geom_stars(data = biod_alloc_stars_prj_sub_re, alpha = 0.9) +
  # stars color fill
  scale_fill_manual(breaks = biod_labels, values = biod_hex, na.value = "white",
                    name = "Biodiversity Index Value",
                    guide = guide_legend(ncol=5,
                                         direction = "horizontal",
                                         title.position = "top",
                                         title.hjust =0.5
                    )) +
  geom_sf(data=bnd0, aes(linetype = bndtype, color = colortype), fill = NA,
          lwd=0.1,show.legend = FALSE) +
  scale_color_manual(values = c("land" = "gray80","ocean"="#ADD8E6"))+
  scale_linetype_manual(values = lineT) +
  coord_sf()+
  mytheme +
  # drop axis labels per reviewer comment
  theme(legend.position="bottom",
        axis.text.x = element_blank(),
        axis.text.y = element_blank(),
        axis.ticks = element_blank()) +
  theme(legend.key = element_rect(fill = "white")) +
  theme(strip.background =element_rect(fill="gray92"))+
  facet_wrap(new_dim ~ ., ncol=2, strip.position="top",
             labeller = label_wrap_gen(multi_line = TRUE,width=110))

p11panel_fn <- file.path(figdir,paste0('fig_11_plants_and_vertebrates_five_threats_',
                                       '_ddh',ddh_vers,
                                       'run',run_vers,'.png'))
ggsave(p11panel_fn,p11panel,dpi=300,width = 8, height = 8, units = "in")
