################################################################################
##
## Table 3: model-predicted risk assignments vs. IUCN assignments
##
################################################################################

species_tab_fn <- file.path(wkdir,'local_data',sprintf("vers%s",ddh_vers), "GBIF_Merged_IUCN_Risks_Tab.csv")
species_tab <- readr::read_csv(species_tab_fn)

if('tprob' %in% names(species_tab)) {
  species_tab <- species_tab %>%
    dplyr::rename(.,pthreat = tprob)
}

# create grouping for table
species_iucn <- species_tab %>%
  dplyr::select(species,irisk) %>%
  mutate(irisklc = ifelse(!is.na(irisk),
                          ifelse(irisk=='1LC','LC','Above LC'),
                              NA))
dim(species_iucn)
table(species_iucn$irisk)
table(species_iucn$irisklc)

# GBIF model predicted risk
# create grouping for table
vcat2 <- c('Land_Amphibians','Land_Birds','Land_Mammals','Land_Reptiles')
species_threats <- species_tab %>%
  # aggregate vertebrate categories to new var
  mutate(loc_spp_grp = ifelse(rcat %in% vcat2,'Land_Vertebrates', rcat)) %>%
  # Land fish to Land other
  mutate(loc_spp_grp = ifelse(loc_spp_grp=='Land_Fish','Land_Other',loc_spp_grp)) %>%
  dplyr::select(lcat,loc_spp_grp,species,pthreat)

table(species_threats$loc_spp_grp)

# merge
species_merged <- merge(species_iucn,species_threats,by='species',all=TRUE)
dim(species_merged)

species_merged2 <- species_merged %>%
  filter(complete.cases(.))

# change labels to model predictions: probability group %
# [0,10] = 0 ...
species_merged2$pthreatgrp = as.character(factor(cut(species_merged2$pthreat,breaks=seq(0,100,by=10),
                                                     labels=paste(seq(0,90,by=10)),
                          include.lowest = TRUE,right=TRUE)))
dim(species_merged2)

# check lowest is included
table(species_merged2$pthreatgrp)
table(species_merged2$irisklc)

# model predictions probability group
species_merged3 <- species_merged2 %>%
  mutate(d_LC = ifelse(irisklc=="LC",1,0),
         d_gtLC = ifelse(irisklc=="Above LC",1,0)) %>%
  dplyr::select(loc_spp_grp,irisk,pthreatgrp,d_LC,d_gtLC) %>%
  dplyr::group_by(pthreatgrp,loc_spp_grp) %>%
  dplyr::summarize(sum_LC = sum(d_LC,na.rm=TRUE),
                   sum_gtLC = sum(d_gtLC,na.rm=TRUE),
                   prcLC = sum(d_LC,na.rm=TRUE) / n(),
                   prcgtLC = sum(d_gtLC,na.rm=TRUE)  / n(),
                   n_species = n()) %>%
  arrange(pthreatgrp)

species_merged4 <- species_merged3 %>%
  dplyr::select(pthreatgrp, loc_spp_grp,prcLC, prcgtLC) %>%
  tidyr::pivot_wider(id_cols = c(pthreatgrp), names_from = loc_spp_grp, values_from = c(prcLC, prcgtLC),
                     names_glue = "{loc_spp_grp}_{.value}",
                     names_sort = TRUE,
                     names_vary="slowest") %>%
  as.data.frame()
  
species_merged4

# format multiply by 100 and 
# 3 digits for publishing
species_merged4b <- species_merged4
species_merged4b[,-1] <- formatC(signif(as.matrix(species_merged4[,-1] *100), digits=3), digits=3, format="fg", flag="#")

# reorder columns
species_merged4b <- species_merged4b %>%
  dplyr::relocate(c(Land_Vertebrates_prcLC,Land_Vertebrates_prcgtLC), .after = pthreatgrp) %>%
  dplyr::relocate(c(Land_Other_prcLC,Land_Other_prcgtLC), .after = Land_Plants_prcgtLC)

# write csv
table3_df_fn <- file.path(tbldir,paste0('Table03_model_predicted_risk_assignments',Sys.Date(),'.csv'))
write.csv(species_merged4b,table3_df_fn,row.names=FALSE)

# gt table
tbl3 <- species_merged4 %>%
  gt::gt() %>%
  gt::tab_header(title = "Model-predicted risk assignments vs. IUCN assignments") %>%
  gt::tab_spanner_delim(
    delim = "_"
  ) %>% 
  gt::fmt_percent(ends_with("LC"), decimals = 1) %>%
  gt::tab_source_note(source_note = "Source: GBIF and IUCN") %>%
  gt::cols_label(pthreatgrp = "Prob. Group",
             ends_with("prcLC") ~ "LC",
             ends_with("prcgtLC") ~ "Above LC"
             )
table3_fn <- file.path(tbldir,paste0('Table03_model_predicted_risk_assignments',Sys.Date(),'.html'))
gt::gtsave(tbl3, table3_fn)
